/*  FreeJ
 *  (c) Copyright 2001 Denis Roio aka jaromil <jaromil@dyne.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <iostream>
#include <assert.h>
#include <context.h>
#include <keyboard.h>
#include <v4l.h>
#include <osd.h>
#include <utils.h>
#include <lubrify.h>
#include <config.h>
#include "filters/filter-list.h"

/* BE WARNED: hires mode is not supported, if you change here
   you get strange results. */
#define LORES 1

#ifdef LORES
#define W 400
#define H 300
#define D 16
#define GW 256
#define GH 256
#else
#define W 800
#define H 600
#define D 16
#define GW 512
#define GH 512
#endif

int main (int argc, char **argv) {
  
  notice("%s version %s [ http://freej.dyne.org ]",PACKAGE,VERSION);
  act("(c)2001 Denis Roio aka jaromil");
  set_debug(1);

  if(argc>1)
    notice("Commandline switches are not supported, refer to file USAGE for help.");
    
  /* turn on VERY VERBOSE debugging of multithreading kernel
     LTI::Kernel::trace = true; */

  /* this is the output context (screen) */
  Context screen(W,H,D,SDL_HWSURFACE|SDL_DOUBLEBUF);
  if(screen.surf==NULL) exit(0);

  /* this is a layer: actually only v4l is implemented */
  V4lGrabber grabber;
  
  /* this is the keyboard listener */
  KbdListener keyb;

  /* this is the On Screen Display */
  Osd osd;

  /* detect v4l grabber layer */
  assert( grabber.detect("/dev/video") );
  
  /* init the v4l grabber */
  assert( grabber.init(&screen,GW,GH) );

  /* center the v4l layer */
  grabber.x = (Uint16)(W-GW)>>1;
  grabber.y = (Uint16)(H-GH)>>1;
  /* --------------------- */

  /* init the keyb listener */
  keyb.init(&screen);
  
  /* update the fps counter every 25 frames */
  screen.set_fps_interval(24);

  /* create and initialize the osd */
  osd.init(&screen);
  osd.active();

  grabber.start();
  keyb.start();

  Layer *lay;

  while(!keyb.quit) {
    /* main loop */
    if(screen.calc_fps()) {
      lay = (Layer *)screen.layers.begin();
      while(lay != NULL) {
	while(!screen.syncwith((LTI::Object *)lay,lay->id_scr));
	lay->cafudda();
	lay = (Layer *)lay->next;
      }
      screen.flip();
    }
  }

  keyb.join();

  osd.credits();
  screen.flip();
  SDL_Delay(5000);
  osd.credits();
  
  grabber.quit = true;
  grabber.join();

  /* this calls all _delete() methods to safely free al memory */
  screen.close();
  
  exit(1);
}
