/*  FreeJ
 *  (c) Copyright 2001 Denis Roio aka jaromil <jaromil@dyne.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <iostream.h>

#include <context.h>
#include <utils.h>
#include <keyboard.h>
#include <filter.h>
#include <filters/filter-list.h>

#define DELAY 300

bool KbdListener::init(Context *context) {
  this->screen = context;
  context->kbd = this;
  layer = (Layer *)context->layers.begin();

  /* TODO ---- when more layers are added this must be fixed */
  layer->id_kbd = context->layers.len()+1000;

  filter = NULL;
  filtersel = 0;

  quit = false;
  return(true);
}

void KbdListener::run() {
  while(!quit) {

    if(SDL_PollEvent(&event)) {
      if(event.type==SDL_KEYDOWN) {

	switch(event.key.keysym.sym) {
	case SDLK_ESCAPE:
	  quit = true;
	  break;
	case SDLK_SPACE:
	  SDL_WM_ToggleFullScreen(screen->surf);
	  break;
	case SDLK_TAB:
	  if(event.key.keysym.mod == KMOD_LCTRL)
	    screen->osd->calibrate();
	  else
	    screen->osd->active();
	  break;
	  /* commented out until layer is just one
	     case SDLK_PAGEUP:
	     if(layer->next!=NULL)
	     layer = (Layer *) layer->next;
	     break;
	     case SDLK_PAGEDOWN:
	     if(layer->prev!=NULL)
	     layer = (Layer *) layer->prev;
	     break; */
	default:
	  if(_context_op(&event.key.keysym)) break;

	  // while(!syncwith((LTI::Object *)layer,layer->id_kbd));
	  /* layer->keypress(&event.key.keysym); */
	  if(filter!=NULL) filter->kbd_input(&event.key.keysym);
	  break;
	}

      } else if(event.type==SDL_QUIT) quit = true;
    }
    SDL_Delay(DELAY);
  }
}

/* manages keys relative to operations on the layer:
   add/delete/move a filter */

bool KbdListener::_context_op(SDL_keysym *keysym) {
  bool newfilt = false;
  Filter *filt = NULL;

  switch(keysym->sym) {

  /* control numbers for adding filters */
  case SDLK_1:
    filt = new Bitcolor;
    newfilt = true;
    break;
  case SDLK_2:
    filt = new Strobe;
    newfilt = true;
    break;
  case SDLK_3:
    filt = new Delaygrab;
    newfilt = true;
    break;
  case SDLK_4:
    filt = new Absdiff;
    newfilt = true;
    break;
  case SDLK_5:
    filt = new Rotozoom;
    newfilt = true;
    break;

  case SDLK_UP:
    if(filter==NULL) return false;
    if(filter->prev==NULL) return false;

    if(keysym->mod == KMOD_RCTRL) {
    /* move filter up in chain */
      while(!syncwith((LTI::Object *)layer,layer->id_kbd));
      if(layer->moveup_filter(filtersel))
	filtersel--;
    } else {
      /* select filter up */
      filter = (Filter *)filter->prev;
      filtersel--;
    }
    return true;
    break;
  case SDLK_DOWN:
    if(filter==NULL) return false;
    if(filter->next==NULL) return false;

    if(keysym->mod == KMOD_RCTRL) {
      /* move filter down in chain */
      while(!syncwith((LTI::Object *)layer,layer->id_kbd));
      if(layer->movedown_filter(filtersel))
	filtersel++;      
    } else {
      /* select filter down */
      filter = (Filter *)filter->next;
      filtersel++;
    }
    return true;
    break;
  case SDLK_INSERT:
    if(filter==NULL) return false;
    /* switch filter on/off */
    filter->active = !filter->active;
    return true;
    break;

  case SDLK_PRINT:
    screen->osd->credits();
    break;

  case SDLK_DELETE:
    if(filter==NULL) return false;
    if(keysym->mod==KMOD_RCTRL) {
      while(!syncwith((LTI::Object *)layer,layer->id_kbd));
      layer->clear_filters();
      filter = NULL;
      filtersel = layer->filters.len();
    } else {
      Filter *tmp = (filter->prev==NULL) ? (Filter *)filter->next : (Filter *)filter->prev;
      while(!syncwith((LTI::Object *)layer,layer->id_kbd));
      layer->del_filter(filtersel);
      filter = tmp;
      filtersel = (filter==NULL) ? 0 : (filtersel>1) ? filtersel-1 : 1;
    }
    return true;
    break;
  default:
    break;
  }

  if(newfilt) {
    while(!syncwith((LTI::Object *)layer,layer->id_kbd));
    newfilt = layer->add_filter(filt);
    filter = filt;
    filtersel = layer->filters.len();
  }
  
  return newfilt;
}
